# Installing Required Software on your Development System

The following instructions need to be followed and executed on your *Development System*, regardless of whether this runs in a virtual machine or not.

### Setting up SSH

Open a terminal window (on your Development System as defined above!) and execute the following command to update the list of packages available for your system:
```bash
sudo apt update
```

Install the openssh package:
```bash
sudo apt install openssh-server
```

In the instructions below replace USER with the specific name of the user you logged in with. 
You can check what name it is by executing the following command to display the value of the USER environment variable:
```bash
echo $USER
```

Test if you can ssh from the development system into itself without providing the password. It is necessary for the remaining steps described below.  
```bash
ssh USER@localhost
```
If you are asked to enter the password, cancel the command (Ctrl+C) and execute the following command to install the needed ssh key:
```bash
ssh-copy-id -i USER@localhost
```
If you don't have an SSH key generated on the development system yet, execute:
```bash
ssh-keygen
```
and repeat the previous step. You should now be able to login to the development system using ```ssh USER@localhost```

### Installing Ansible

Some of the WARA PS software is installed using [Ansible Playbooks](https://docs.ansible.com/ansible/latest/index.html).

If you use Ubuntu 18.04, you first need to do the following in order to get access to a repository containing the Ansible software:

```bash
## sudo install --reinstall ca-certificates # Had to do this for the apt-add-repository to work
sudo apt install software-properties-common
sudo apt-add-repository --yes --update ppa:ansible/ansible
```

Regardless of Ubuntu version, continue as follows:

```bash
sudo apt install ansible
```

### Installing software using Ansible Playbooks

Clone our playbooks repo to get access to the actual playbooks for WARA PS:

```bash
sudo apt install git
git clone https://gitlab.liu.se/lrs/lrs_playbooks.git
```

Run the `wara_develop` playbook as follows to install and update the WARA PS developer software. It can take several minutes to complete.

- In the command below the value of the environment variable specifying the login name is used. You don't have to replace it.

- But make sure you replace the last argument with the actual password of the root/administrator on the development system.

```bash
cd lrs_playbooks
git pull
ansible-playbook -i hosts playbooks/wara_develop.yml -e user=${USER} -e install_user=${USER} -e ansible_sudo_pass=******
```

### Finishing Up

Add the following commands at the end of the ```~/.bashrc``` file on your development system:
```bash
export WORLD_ORIGIN_LAT=57.7605573519 
export WORLD_ORIGIN_LON=16.6827607783
export WORLD_ORIGIN_ELEVATION=29.8

#source /opt/ros/melodic/setup.bash
source ${HOME}/wara_ws/devel/setup.bash
export PATH=${HOME}/lrs_devenv_common/script:${PATH}
```

Then log in and log out to the development system for the docker commands to work.  Test this with:

```bash
docker ps
```
It should show:
```bash
tompe@philo:~$ docker ps
CONTAINER ID        IMAGE               COMMAND             CREATED             STATUS              PORTS               NAMES
tompe@philo:~$ 
```

At this point your system now has the necessary software installed.

You can use the following commands on the development system to test the 3D graphics hardware and driver setup on your system:

```bash
glxgears
glmark2
```

If the above commands execute without errors you can test that Gazebo
works correctly on your development system:

```bash
roslaunch lrs_wara_gazebo gazebo.launch
roslaunch lrs_wara_gazebo dji_to_gazebo.launch name:=dji0 with_laser:=true x:=5
```

All files to run these tests should be available in lrs_wara_gazebo
which is installed in wara_ws by the Playbook above. The first command
starts Gazebo and the second command adds a model of a DJI M100 to the
Gazebo world.
