# Local ROS Installation and LRS

Here we describe how to use a local ROS installation and interface
with the LRS Modules available through docker. The intention is that
this should be enough for a person knowledgeable about ROS to start
developing their own functionality and interface with the available ROS
modules.

See the following for en overview of the available functionality:

- https://aiics.waraps.org/index.php/lrs-lrs_jupyter-md-djisdk-md/
- https://aiics.waraps.org/index.php/lrs-lrs_jupyter-md-single_agent-md/
- https://aiics.waraps.org/index.php/lrs-lrs_jupyter-md-single_agent_tst-md/


## Table of Content

- [Preparations](#preparations)
- [Scripts](#scripts)
- [How to start different parts](#how-to-start-different-parts)
- [Flying with Gamepad](#flying-with-gamepad)

## Preparation

Get the lrs_devenv_common repo:
```bash
git clone git@gitlab.liu.se:lrs/lrs_devenv_common.git
```
or if you need to specify the key:
```bash
GIT_SSH_COMMAND='ssh -i ~/.ssh/wara_deploy -o IdentitiesOnly=yes' git clone git@gitlab.liu.se:lrs/waraps.git
```

Put the script directory of the lrs_devenv_common repo in your PATH plus add
some variables that are needed. For example in .bashrc:

```bash
export PATH="...../lrs_devenv_common/script:$PATH"

export WORLD_ORIGIN_LAT=57.7605573519 
export WORLD_ORIGIN_LON=16.6827607783
export WORLD_ORIGIN_ELEVATION=29.8
```

If you have gotten a lot of running containers that interfere with
each other you can stop allrunning containers with:

```bash
docker stop $(docker container ls -q)
```

And to clean up:
```bash
docker system prune
```


## Scripts

You now have the following scripts in your path:

### wexec

Execute commands in a running docker container.
```bash
#!/bin/bash

docker exec -it lrsdevel "$@"
```

### wgrun

Start a docker container named lrsdevel and run the commands there. Use this if you have NVIDIA graphics.
```bash
#!/bin/bash

if [[ $(lsb_release -rs) == "18.04" ]]; then
docker run --gpus all \
  --network host \
  --security-opt apparmor:unconfined \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel "$@"
fi

if [[ $(lsb_release -rs) == "20.04" ]]; then
docker run --gpus all \
  --network host \
  --security-opt apparmor:unconfined \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel "$@"
fi
```


### wrun

Start a docker container named lrsdevel and run the command there. Use if you do not have NVIDIA graphics.
```bash
#!/bin/bash

if [[ $(lsb_release -rs) == "18.04" ]]; then
docker run \
  --network host \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel "$@"
fi

if [[ $(lsb_release -rs) == "20.04" ]]; then
docker run \
  --network host \
  --ulimit nofile=1024 \
  --ipc=host \
  -v /tmp/.X11-unix:/tmp/.X11-unix \
  -v /tmp:/exttmp \
  -e DISPLAY=unix${DISPLAY}\
  -e ROS_HOSTNAME=localhost \
  --rm -it --name lrsdevel gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel "$@"
fi
```


### wpull

Pull the image.
```bash
#!/bin/bash

if [[ $(lsb_release -rs) == "18.04" ]]; then
    echo "Pulling gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel"
    docker pull gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-melodic:devel
fi

if [[ $(lsb_release -rs) == "20.04" ]]; then
    echo "Pulling gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel"
    docker pull gitlab.liu.se:5000/lrs/devenv_docker_images/lrs-noetic:devel
fi
```


## How to start different parts

### Pull the Image
Start by puling the image:
```bash
wpull
```

### Start a ros core and the docker container:

Either run the roscore locally or in the docker container.

Local roscore:
```bash
roscore
wgrun bash
```

Roscore in docker container:
```bash
wgrun roscore
```

### Loading config files for units

Assume we want to use unit /dji0 in Gränsö. The do:
```bash
wexec roslaunch lrs_launch config.launch location:=granso no_airspace_booking:=true ns:=/dji0
```


### Start gazebo using docker (optionally)

If you have a NVIDIA graphics card and support for that in docker then
you can if needed start gazebo (and if the container was started with wgrun):

```bash
wexec roslaunch lrs_wara_gazebo gazebo.launch
```

Load the dji model in gazebo:

```bash
wexec roslaunch lrs_wara_gazebo dji_to_gazebo.launch name:=dji0 with_laser:=true x:=0
```


### dji 0

```bash
wexec roslaunch lrs_launch djisimtst.launch dynamics:=true publish_pose:=true mpc:=true ns:=/dji0
```
In ONE roscore mode the following holds:
- opunit.launch will start a program thet convert from geopose to pose for all units.
- So the flag publish_pose should only be set to true if you run opunit.launch.

In multiple roscore mode only geopose is sent to the op unit so there
we can choose freely if we want to generate the pose or not.


### Start tstdisplayserver

```bash
wexec rosrun lrs_util_tst tstdisplayserver.py __ns:=/dji0
```

### RVIZ

Gränsö:
```bash
wexec roslaunch lrs_dji_sim rviz_granso.launch
```

Geneneral, uses the location given to config. Currently only Gränsö ortho photo installed.
```bash
wexec roslaunch lrs_launch rviz_outside.launch
```

### Test the dji0

On the local machine do:

Watch the geopose:
```bash
rostopic echo /dji0/geopose
```

Takeoff
```bash
wexec rosrun lrs_tst tstcommand.py --takeoff --exec __ns:=/dji0 --disp --op /dji0
```

Flyto
```bash
wexec rosrun lrs_tst tstcommand.py --fly -x 100 -y 100 -z 12 --speed 3 --exec __ns:=/dji0 --disp --op /dji0
```

## Flying with Gamepad

For this case it is more convenient to runt the roscore on the local machine:
```bash
roscore
```

And start the container as:
```bash
wgrun bash
```

On the local machine run the joy ros node:

Set the variable:
```bash
echo $JOY_EXTRA0 
usb-Logitech_Wireless_Gamepad_F710_2CAE9E79-joystick
```

To get the id for the Gamepad do:
```bash
tompe@philo:~$ ls /dev/input/by-id/
usb-046d_C922_Pro_Stream_Webcam_B16031EF-event-if00
usb-CNFEH73M102010001C32_Integrated_Webcam_HD-event-if00
usb-ELAN_Touchscreen-event-if00
usb-HP_HP_USB_1000dpi_Laser_Mouse__LiteON_-event-mouse
usb-HP_HP_USB_1000dpi_Laser_Mouse__LiteON_-mouse
usb-Logitech_USB_Keyboard-event-if01
usb-Logitech_USB_Keyboard-event-kbd
usb-Logitech_Wireless_Gamepad_F710_2CAE9E79-event-joystick
usb-Logitech_Wireless_Gamepad_F710_2CAE9E79-joystick
```

Start the ROS driver for a joystick:
```bash
rosrun joy joy_node _dev:=/dev/input/by-id/${JOY_EXTRA0} _deadzone:=0.05 _autorepeat_rate:=50 _coalesce_interval:=0.1 __ns:=/extra0
```

Setup the simulation:
```bash
wexec roslaunch lrs_launch config.launch location:=granso no_airspace_booking:=true ns:=/dji0
wexec roslaunch lrs_launch djisimtst.launch dynamics:=true publish_pose:=true mpc:=true inair:=true ns:=/dji0
```

Start the joystick controller:
```bash
wexec rosrun lrs_dji_sim joypose.py __ns:=/dji0
```


Rviz:
```bash
wexec roslaunch lrs_launch rviz_outside.launch
```

On the GamePad:
- Green Button: Manual Mode, fly with gamepad
- Red Button: Auto
